@@ -1,4 +1,4 @@
-# Copyright 2020–2021 Cirq on IQM developers
+# Copyright 2020–2022 Cirq on IQM developers
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,6 @@ Circuit sampler that executes quantum circuits on an IQM quantum computer.
 from __future__ import annotations
 
 import json
-from typing import Optional
 
 import cirq
 import numpy as np
@@ -67,13 +66,23 @@ class IQMSampler(cirq.work.Sampler):
         settings: Settings for the quantum computer
         device: Quantum architecture to execute the circuit on
         qubit_mapping: Injective dictionary that maps logical qubit names to physical qubit names
-        username: Username, if required by the IQM Cortex server. This can also be set in the IQM_SERVER_USERNAME
-                  environment variable.
-        api_key: API key, if required by the IQM Cortex server. This can also be set in the IQM_SERVER_API_KEY
-                 environment variable.
+
+    Keyword Args:
+        auth_server_url: URL of user authentication server, if required by the IQM Cortex server.
+            This can also be set in the IQM_AUTH_SERVER environment variable.
+        username: Username, if required by the IQM Cortex server.
+            This can also be set in the IQM_AUTH_USERNAME environment variable.
+        password: Password, if required by the IQM Cortex server.
+            This can also be set in the IQM_AUTH_PASSWORD environment variable.
     """
-    def __init__(self, url: str, settings: str, device: IQMDevice, qubit_mapping: dict[str, str] = None, *,
-                 username: Optional[str] = None, api_key: Optional[str] = None):
+    def __init__(
+            self,
+            url: str,
+            settings: str,
+            device: IQMDevice,
+            qubit_mapping: dict[str, str] = None,
+            **user_auth_args  # contains keyword args auth_server_url, username and password
+    ):
         settings_json = json.loads(settings)
 
         if qubit_mapping is None:
@@ -89,10 +98,15 @@ class IQMSampler(cirq.work.Sampler):
         if diff:
             raise ValueError(f'The physical qubits {diff} in the qubit mapping are not defined in the settings.')
 
-        self._client = IQMClient(url, settings=settings_json, username=username, api_key=api_key)
+        self._client = IQMClient(url, settings_json, **user_auth_args)
         self._device = device
         self._qubit_mapping = qubit_mapping
 
+    def close_client(self):
+        """Close IQMClient's session with the user authentication server. Discard the client."""
+        self._client.close()
+        self._client = None
+
     def run_sweep(
             self,
             program: cirq.Circuit,
@@ -117,14 +131,11 @@ class IQMSampler(cirq.work.Sampler):
         if len(sweeps) > 1 or len(sweeps[0].keys) > 0:
             raise NotImplementedError('Sweeps are not supported')
 
-        if program.device is cirq.UNCONSTRAINED_DEVICE:
-            # verify that qubit_mapping covers all qubits in the circuit
-            circuit_qubits = set(qubit.name for qubit in program.all_qubits())
-            diff = circuit_qubits - set(self._qubit_mapping)
-            if diff:
-                raise ValueError(f'The qubits {diff} are not found in the provided qubit mapping.')
-        elif program.device != self._device:
-            raise ValueError('The devices of the given circuit and of the sampler are not the same.')
+        # verify that qubit_mapping covers all qubits in the circuit
+        circuit_qubits = set(qubit.name for qubit in program.all_qubits())
+        diff = circuit_qubits - set(self._qubit_mapping)
+        if diff:
+            raise ValueError(f'The qubits {diff} are not found in the provided qubit mapping.')
 
         # apply qubit_mapping
         qubit_map = {cirq.NamedQubit(k): cirq.NamedQubit(v) for k, v in self._qubit_mapping.items()}
@@ -161,4 +172,4 @@ class IQMSampler(cirq.work.Sampler):
         job_id = self._client.submit_circuit(iqm_circuit, qubit_mapping, repetitions)
         results = self._client.wait_for_results(job_id)
         measurements = {k: np.array(v) for k, v in results.measurements.items()}
-        return study.Result(params=resolver.ParamResolver(), measurements=measurements)
+        return study.ResultDict(params=resolver.ParamResolver(), measurements=measurements)
