@@ -1,4 +1,4 @@
-# Copyright 2020–2021 Cirq on IQM developers
+# Copyright 2020–2022 Cirq on IQM developers
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ from iqm_client.iqm_client import (IQMClient, RunResult, RunStatus,
                                    SingleQubitMapping)
 from mockito import ANY, mock, when
 
-from cirq_iqm import Adonis, Valkmusa
+from cirq_iqm import Adonis
 from cirq_iqm.iqm_sampler import IQMSampler, serialize_qubit_mapping
 
 
@@ -68,18 +68,16 @@ def test_run_sweep_executes_circuit(adonis_sampler, circuit):
 
 
 def test_credentials_are_passed_to_client(settings_dict):
-    username = 'a fake username'
-    api_key = 'a fake api key'
-    sampler = IQMSampler('some url', json.dumps(settings_dict), Adonis(), None, username=username, api_key=api_key)
-    assert sampler._client._credentials[0] == username
-    assert sampler._client._credentials[1] == api_key
-
-
-def test_circuit_with_incorrect_device(adonis_sampler):
-    circuit = cirq.Circuit(device=Valkmusa())
-    with pytest.raises(ValueError, match='devices .* not the same'):
-        adonis_sampler.run(circuit)
-
+    user_auth_args = {
+        'auth_server_url': 'https://fake.auth.server.com',
+        'username': 'fake-username',
+        'password': 'fake-password',
+    }
+    with when(IQMClient)._update_tokens():
+        sampler = IQMSampler('http://url', json.dumps(settings_dict), Adonis(), None, **user_auth_args)
+    assert sampler._client._credentials.auth_server_url == user_auth_args['auth_server_url']
+    assert sampler._client._credentials.username == user_auth_args['username']
+    assert sampler._client._credentials.password == user_auth_args['password']
 
 def test_non_injective_qubit_mapping(base_url, settings_dict, qubit_mapping):
     qubit_mapping['q2 log.'] = 'QB1'
